@@ -27,13 +27,12 @@
 
 #include <libtracker-common/tracker-common.h>
 
-#include <libtracker-db/tracker-db-dbus.h>
-#include <libtracker-db/tracker-db-interface-sqlite.h>
-#include <libtracker-db/tracker-db-manager.h>
-
 #include "tracker-class.h"
 #include "tracker-data-manager.h"
 #include "tracker-data-query.h"
+#include "tracker-db-dbus.h"
+#include "tracker-db-interface-sqlite.h"
+#include "tracker-db-manager.h"
 #include "tracker-ontologies.h"
 #include "tracker-sparql-query.h"
 
@@ -66,11 +65,11 @@ tracker_data_query_rdf_type (gint id)
 		 * function is called fairly often) */
 
 		ret = g_ptr_array_sized_new (20);
-		while (tracker_db_cursor_iter_next (cursor, &error)) {
+		while (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			const gchar *class_uri;
 			TrackerClass *cl;
 
-			class_uri = tracker_db_cursor_get_string (cursor, 0);
+			class_uri = tracker_db_cursor_get_string (cursor, 0, NULL);
 			cl = tracker_ontologies_get_class_by_uri (class_uri);
 			if (!cl) {
 				g_critical ("Unknown class %s", class_uri);
@@ -117,7 +116,7 @@ tracker_data_query_resource_id (const gchar *uri)
 	}
 
 	if (cursor) {
-		if (tracker_db_cursor_iter_next (cursor, &error)) {
+		if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			id = tracker_db_cursor_get_int (cursor, 0);
 		}
 
