@@ -47,6 +47,7 @@ struct _TrackerPropertyPrivate {
 
 	TrackerPropertyType  data_type;
 	TrackerClass   *domain;
+	TrackerClass   *domain_index;
 	TrackerClass   *range;
 	gint           weight;
 	gint           id;
@@ -62,8 +63,10 @@ struct _TrackerPropertyPrivate {
 	gboolean       db_schema_changed;
 	gboolean       writeback;
 	gchar         *default_value;
+	gboolean       is_new_domain_index;
 
 	GArray        *super_properties;
+	GArray        *domain_indexes;
 };
 
 static void property_finalize     (GObject      *object);
@@ -133,6 +136,7 @@ tracker_property_init (TrackerProperty *property)
 	priv->transient = FALSE;
 	priv->multiple_values = TRUE;
 	priv->super_properties = g_array_new (TRUE, TRUE, sizeof (TrackerProperty *));
+	priv->domain_indexes = g_array_new (TRUE, TRUE, sizeof (TrackerClass *));
 
 	/* Make GET_PRIV working */
 	property->priv = priv;
@@ -162,6 +166,7 @@ property_finalize (GObject *object)
 	}
 
 	g_array_free (priv->super_properties, TRUE);
+	g_array_free (priv->domain_indexes, TRUE);
 
 	g_free (priv->default_value);
 
@@ -271,6 +276,21 @@ tracker_property_get_domain (TrackerProperty *property)
 	return priv->domain;
 }
 
+TrackerClass **
+tracker_property_get_domain_indexes (TrackerProperty *property)
+{
+	TrackerPropertyPrivate *priv;
+
+	/* Removed for performance:
+	 g_return_val_if_fail (TRACKER_IS_PROPERTY (property), NULL); */
+
+	g_return_val_if_fail (property != NULL, NULL);
+
+	priv = GET_PRIV (property);
+
+	return (TrackerClass ** ) priv->domain_indexes->data;
+}
+
 TrackerClass *
 tracker_property_get_range (TrackerProperty *property)
 {
@@ -370,6 +390,18 @@ tracker_property_get_is_new (TrackerProperty *property)
 	return priv->is_new;
 }
 
+gboolean
+tracker_property_get_is_new_domain_index (TrackerProperty *property)
+{
+	TrackerPropertyPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_PROPERTY (property), FALSE);
+
+	priv = GET_PRIV (property);
+
+	return priv->is_new_domain_index;
+}
+
 gboolean
 tracker_property_get_writeback (TrackerProperty *property)
 {
@@ -529,6 +561,60 @@ tracker_property_set_domain (TrackerProperty *property,
 	}
 }
 
+void
+tracker_property_add_domain_index (TrackerProperty *property,
+                                   TrackerClass    *value)
+{
+	TrackerPropertyPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_PROPERTY (property));
+	g_return_if_fail (TRACKER_IS_CLASS (value));
+
+	priv = GET_PRIV (property);
+
+	g_array_append_val (priv->domain_indexes, value);
+}
+
+void
+tracker_property_del_domain_index (TrackerProperty *property,
+                                   TrackerClass    *value)
+{
+	TrackerClass **classes;
+	TrackerPropertyPrivate *priv;
+	gint i = 0, found = -1;
+
+	g_return_if_fail (TRACKER_IS_PROPERTY (property));
+	g_return_if_fail (TRACKER_IS_CLASS (value));
+
+	priv = GET_PRIV (property);
+
+	classes = (TrackerClass **) priv->domain_indexes->data;
+	while (*classes) {
+		if (*classes == value) {
+			found = i;
+			break;
+		}
+		i++;
+		classes++;
+	}
+
+	if (found != -1) {
+		g_array_remove_index (priv->domain_indexes, found);
+	}
+}
+
+void
+tracker_property_reset_domain_indexes (TrackerProperty *property)
+{
+	TrackerPropertyPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_PROPERTY (property));
+
+	priv = GET_PRIV (property);
+	g_array_free (priv->domain_indexes, TRUE);
+	priv->domain_indexes = g_array_new (TRUE, TRUE, sizeof (TrackerClass *));
+}
+
 void
 tracker_property_set_secondary_index (TrackerProperty *property,
                                       TrackerProperty *value)
@@ -636,6 +722,19 @@ tracker_property_set_is_new (TrackerProperty *property,
 	priv->is_new = value;
 }
 
+void
+tracker_property_set_is_new_domain_index (TrackerProperty *property,
+                                          gboolean         value)
+{
+	TrackerPropertyPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_PROPERTY (property));
+
+	priv = GET_PRIV (property);
+
+	priv->is_new_domain_index = value;
+}
+
 void
 tracker_property_set_writeback (TrackerProperty *property,
                                 gboolean         value)
