@@ -34,32 +34,14 @@
 #include <time.h>
 #include <inttypes.h>
 
+#include <glib/gi18n-lib.h>
 #include <glib-object.h>
 #include <gio/gio.h>
 
 #include <sqlite3.h>
 
-#include <camel/camel-mime-message.h>
-#include <camel/camel-i18n.h>
-#include <camel/camel-store.h>
-#include <camel/camel-folder.h>
+#include <camel/camel.h>
 #include <camel/camel-db.h>
-#include <camel/camel-offline-store.h>
-#include <camel/camel-session.h>
-#include <camel/camel-url.h>
-#include <camel/camel-stream.h>
-#include <camel/camel-stream-mem.h>
-#include <camel/camel-multipart.h>
-#include <camel/camel-multipart-encrypted.h>
-#include <camel/camel-multipart-signed.h>
-#include <camel/camel-medium.h>
-#include <camel/camel-gpg-context.h>
-#include <camel/camel-smime-context.h>
-#include <camel/camel-string-utils.h>
-#include <camel/camel-stream-filter.h>
-#include <camel/camel-stream-null.h>
-#include <camel/camel-mime-filter-charset.h>
-#include <camel/camel-mime-filter-windows.h>
 
 #include <mail/mail-config.h>
 #include <mail/mail-session.h>
@@ -75,8 +57,7 @@
 #include <e-util/e-account-utils.h>
 #endif
 
-#include <libtracker-client/tracker.h>
-#include <libtracker-client/tracker-sparql-builder.h>
+#include <libtracker-sparql/tracker-sparql.h>
 
 #include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-ontologies.h>
@@ -96,7 +77,7 @@
  * reads, never writes). We hope that's sufficient for not having to get our
  * code involved in Camel's cruel inneryard of having to lock the db_r ptr. */
 
-#define TRACKER_SERVICE                         "org.freedesktop.Tracker1"
+#define TRACKER_SERVICE                 "org.freedesktop.Tracker1"
 
 #define NIE_DATASOURCE                  TRACKER_NIE_PREFIX "DataSource"
 #define RDF_PREFIX                      TRACKER_RDF_PREFIX
@@ -125,13 +106,15 @@ G_DEFINE_TYPE (TrackerEvolutionPlugin, tracker_evolution_plugin, TRACKER_TYPE_MI
  * during registration of accounts and folders). I know this is cruel. I know. */
 
 typedef struct {
-	TrackerClient *client;
+	TrackerSparqlConnection *connection;
 	gchar *sparql;
 	gboolean commit;
 	gint prio;
 	GMutex *mutex;
 	GCond *cond;
 	gboolean has_happened;
+	gpointer pool;
+	gboolean dont_free;
 } PoolItem;
 
 typedef struct {
@@ -185,9 +168,9 @@ typedef struct {
 	GHashTable *registered_stores;
 	GList *registered_clients;
 	EAccountList *accounts;
-	TrackerClient *client;
+	TrackerSparqlConnection *connection;
 	DBusGProxy *dbus_proxy;
-	DBusGConnection *connection;
+	DBusGConnection *dbusconnection;
 	time_t last_time;
 	gboolean resuming, paused;
 	guint total_popped, of_total;
@@ -301,8 +284,16 @@ get_email_and_fullname (const gchar *line, gchar **email, gchar **fullname)
 static void
 folder_registry_free (FolderRegistry *registry)
 {
+#ifdef HAVE_EDS_2_31_2
+	g_signal_handler_disconnect (registry->folder, registry->hook_info->hook_id);
+#else
 	camel_object_remove_event (registry->folder, registry->hook_info->hook_id);
+#endif
+#ifdef HAVE_EDS_2_31_2
+	g_object_unref (registry->folder);
+#else
 	camel_object_unref (registry->folder);
+#endif
 	g_free (registry->hook_info->account_uri);
 	g_slice_free (OnSummaryChangedInfo, registry->hook_info);
 	g_slice_free (FolderRegistry, registry);
@@ -319,7 +310,11 @@ folder_registry_new (const gchar *account_uri,
 	registry->hook_info->account_uri = g_strdup (account_uri);
 	registry->hook_info->self = self; /* weak */
 	registry->hook_info->hook_id = 0;
+#ifdef HAVE_EDS_2_31_2
+	g_object_ref (folder);
+#else
 	camel_object_ref (folder);
+#endif
 	registry->folder = folder;
 
 	return registry;
@@ -331,7 +326,7 @@ free_pool_item (gpointer data, gpointer user_data)
 {
 	PoolItem *item = data;
 	g_free (item->sparql);
-	g_object_unref (item->client);
+	g_object_unref (item->connection);
 	g_slice_free (PoolItem, item);
 }
 
@@ -339,6 +334,7 @@ static void
 thread_pool_exec (gpointer data, gpointer user_data)
 {
 	ThreadPool *pool = user_data;
+	PoolItem *item;
 	gboolean dying;
 
 	g_mutex_lock (pool->mutex);
@@ -346,26 +342,39 @@ thread_pool_exec (gpointer data, gpointer user_data)
 	pool->items = g_list_remove (pool->items, data);
 	g_mutex_unlock (pool->mutex);
 
+
 	if (!dying)
 		pool->func (data, pool->cancel);
 
-	pool->freeup (data, pool->cancel);
+	item = data;
+	if (!item->dont_free)
+		pool->freeup (data, pool->cancel);
 }
 
-
-static void 
-reply_void (GError *error, gpointer  user_data)
+static void
+reply_void (GObject *source_object,
+            GAsyncResult *result,
+            gpointer user_data)
 {
 	PoolItem *item = user_data;
+	ThreadPool *pool = item->pool;
+	GError *error = NULL;
+
+	tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (source_object),
+	                                         result, &error);
 
 	if (error) {
 		g_debug ("Tracker plugin: Error updating data: %s\n", error->message);
+		g_error_free (error);
 	}
 
 	g_mutex_lock (item->mutex);
 	g_cond_broadcast (item->cond);
 	item->has_happened = TRUE;
 	g_mutex_unlock (item->mutex);
+
+	if (item->dont_free)
+		pool->freeup (item, pool->cancel);
 }
 
 static void
@@ -383,10 +392,15 @@ exec_update (gpointer data, gpointer user_data)
 	item->has_happened = FALSE;
 
 	if (item->commit) {
-		tracker_resources_batch_commit_async (item->client, reply_void, item);
+		/* tracker_resources_batch_commit_async (item->client, reply_void, item); */
 	} else {
-		tracker_resources_batch_sparql_update_async (item->client, item->sparql,
-		                                             reply_void, item);
+
+		tracker_sparql_connection_update_async (item->connection,
+		                                        item->sparql,
+		                                        G_PRIORITY_DEFAULT,
+		                                        NULL,
+		                                        reply_void, item);
+
 	}
 
 	g_mutex_lock (item->mutex);
@@ -395,6 +409,7 @@ exec_update (gpointer data, gpointer user_data)
 		g_get_current_time (&val);
 		g_time_val_add (&val, 5 * 1000000); /* 5 seconds worth of patience */
 		no_patience = g_cond_timed_wait (item->cond, item->mutex, &val);
+		item->dont_free = !no_patience;
 	}
 	g_mutex_unlock (item->mutex);
 
@@ -437,9 +452,11 @@ thread_pool_new (GFunc func, GFunc freeup, GCompareDataFunc sorter)
 static void
 thread_pool_push (ThreadPool *pool, gpointer item, gpointer user_data)
 {
+	g_mutex_lock (pool->mutex);
 	pool->items = g_list_prepend (pool->items, item);
 	if (!pool->dying)
 		g_thread_pool_push (pool->pool, item, user_data);
+	g_mutex_unlock (pool->mutex);
 }
 
 static gpointer
@@ -474,15 +491,17 @@ send_sparql_update (TrackerEvolutionPlugin *self, const gchar *sparql, gint prio
 {
 	TrackerEvolutionPluginPrivate *priv = TRACKER_EVOLUTION_PLUGIN_GET_PRIVATE (self);
 
-	if (priv->client) {
+	if (priv->connection) {
 		PoolItem *item = g_slice_new (PoolItem);
 
 		if (!sparql_pool)
 			sparql_pool = thread_pool_new (exec_update, free_pool_item, pool_sort_func);
 
+		item->pool = sparql_pool;
+		item->dont_free = FALSE;
 		item->prio = prio;
 		item->commit = FALSE;
-		item->client = g_object_ref (priv->client);
+		item->connection = g_object_ref (priv->connection);
 		item->sparql = g_strdup (sparql);
 
 		thread_pool_push (sparql_pool, item, NULL);
@@ -494,7 +513,7 @@ send_sparql_commit (TrackerEvolutionPlugin *self, gboolean update)
 {
 	TrackerEvolutionPluginPrivate *priv = TRACKER_EVOLUTION_PLUGIN_GET_PRIVATE (self);
 
-	if (priv->client) {
+	if (priv->connection) {
 		PoolItem *item = g_slice_new (PoolItem);
 
 		if (update) {
@@ -513,9 +532,11 @@ send_sparql_commit (TrackerEvolutionPlugin *self, gboolean update)
 		if (!sparql_pool)
 			sparql_pool = thread_pool_new (exec_update, free_pool_item, pool_sort_func);
 
+		item->pool = sparql_pool;
+		item->dont_free = FALSE;
 		item->prio = 0;
 		item->commit = TRUE;
-		item->client = g_object_ref (priv->client);
+		item->connection = g_object_ref (priv->connection);
 		item->sparql = NULL;
 
 		thread_pool_push (sparql_pool, item, NULL);
@@ -529,7 +550,7 @@ add_contact (TrackerSparqlBuilder *sparql, const gchar *predicate, const gchar *
 
 	get_email_and_fullname (value, &email, &fullname);
 
-	email_uri = tracker_uri_printf_escaped ("mailto:%s", email);
+	email_uri = tracker_sparql_escape_uri_printf ("mailto:%s", email);
 
 	tracker_sparql_builder_subject_iri (sparql, email_uri);
 	tracker_sparql_builder_predicate (sparql, "rdf:type");
@@ -747,6 +768,7 @@ on_folder_summary_changed (CamelFolder *folder,
 
 	summary = folder->summary;
 	em_uri = em_uri_from_camel (account_uri);
+	em_uri [strlen (em_uri) - 1] = '\0';
 
 	merged = g_ptr_array_new ();
 
@@ -813,10 +835,10 @@ on_folder_summary_changed (CamelFolder *folder,
 			/* This is not a path but a URI, don't use the
 			 * OS's directory separator here */
 
-			uri = g_strdup_printf ("%s%s/%s",
-			                       em_uri,
-			                       camel_folder_get_full_name (folder),
-			                       uid);
+			uri = tracker_sparql_escape_uri_printf ("%s/%s/%s",
+			                                        em_uri,
+			                                        camel_folder_get_full_name (folder),
+			                                        uid);
 
 			sparql = tracker_sparql_builder_new_update ();
 
@@ -894,17 +916,17 @@ on_folder_summary_changed (CamelFolder *folder,
 		GString *sparql = g_string_new ("");
 
 		for (i = 0; i< changes->uid_removed->len; i++) {
+			gchar *uri;
 
 			/* This is not a path but a URI, don't use the OS's
 			 * directory separator here */
+			uri = tracker_sparql_escape_uri_printf ("%s/%s/%s",
+			                                        em_uri,
+			                                        camel_folder_get_full_name (folder),
+			                                        (char*) changes->uid_removed->pdata[i]);
 
-			g_string_append_printf (sparql, "DELETE FROM <%s%s/%s> { <%s%s/%s> a rdfs:Resource }\n ",
-			                        em_uri,
-			                        camel_folder_get_full_name (folder),
-			                        (char*) changes->uid_removed->pdata[i],
-			                        em_uri,
-			                        camel_folder_get_full_name (folder),
-			                        (char*) changes->uid_removed->pdata[i]);
+			g_string_append_printf (sparql, "DELETE FROM <%s> { <%s> a rdfs:Resource }\n ", uri, uri);
+			g_free (uri);
 		}
 
 		send_sparql_update (info->self, sparql->str, 100);
@@ -937,6 +959,7 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 	}
 
 	em_uri = em_uri_from_camel (account_uri);
+	em_uri [strlen (em_uri) - 1] = '\0';
 
 	while (iter) {
 		guint count = 0;
@@ -1017,8 +1040,7 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 
 				folder = g_hash_table_lookup (priv->cached_folders, iter->full_name);
 
-				uri =  g_strdup_printf ("%s%s/%s", em_uri,
-				                        iter->full_name, uid);
+				uri = tracker_sparql_escape_uri_printf ("%s/%s/%s", em_uri, iter->full_name, uid);
 
 				if (!sparql) {
 					sparql = tracker_sparql_builder_new_update ();
@@ -1141,7 +1163,10 @@ introduce_store_deal_with_deleted (TrackerEvolutionPlugin *self,
 	sqlite3_stmt *stmt = NULL;
 	CamelDB *cdb_r;
 	guint i, ret;
-	gchar *em_uri = em_uri_from_camel (account_uri);
+	gchar *em_uri;
+
+	em_uri = em_uri_from_camel (account_uri);
+	em_uri [strlen (em_uri) - 1] = '\0';
 
 	query = sqlite3_mprintf ("SELECT uid, mailbox "
 	                         "FROM Deletes "
@@ -1183,8 +1208,9 @@ introduce_store_deal_with_deleted (TrackerEvolutionPlugin *self,
 			/* This is not a path but a URI, don't use the OS's
 			 * directory separator here */
 
-			g_ptr_array_add (subjects_a, g_strdup_printf ("%s%s/%s", em_uri,
-			                                              mailbox, uid));
+			g_ptr_array_add (subjects_a,
+			                 tracker_sparql_escape_uri_printf ("%s/%s/%s", em_uri,
+			                                                   mailbox, uid));
 
 			if (count > 100) {
 				more = TRUE;
@@ -1245,7 +1271,6 @@ get_last_deleted_time (TrackerEvolutionPlugin *self)
 			EAccount *account = (EAccount *) e_iterator_get (it);
 			CamelProvider *provider;
 			CamelStore *store;
-			CamelException ex;
 			char *uri;
 			CamelDB *cdb_r;
 			sqlite3_stmt *stmt = NULL;
@@ -1253,13 +1278,10 @@ get_last_deleted_time (TrackerEvolutionPlugin *self)
 			guint ret = SQLITE_OK;
 			guint64 latest = smallest;
 
-			camel_exception_init (&ex);
-
 			if (!account->enabled || !(uri = account->source->url))
 				continue;
 
 			if (!(provider = camel_provider_get(uri, NULL))) {
-				camel_exception_clear (&ex);
 				continue;
 			}
 
@@ -1267,8 +1289,7 @@ get_last_deleted_time (TrackerEvolutionPlugin *self)
 				continue;
 			}
 
-			if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
-				camel_exception_clear (&ex);
+			if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, NULL))) {
 				continue;
 			}
 
@@ -1323,9 +1344,15 @@ register_on_get_folder (gchar *uri, CamelFolder *folder, gpointer user_data)
 		goto not_ready;
 	}
 
+#ifdef HAVE_EDS_2_31_2
+	hook_id = g_signal_connect (folder, "changed",
+				    G_CALLBACK (on_folder_summary_changed),
+				    registry->hook_info);
+#else
 	hook_id = camel_object_hook_event (folder, "folder_changed",
 	                                   CAMEL_CALLBACK (on_folder_summary_changed),
 	                                   registry->hook_info);
+#endif
 	registry->hook_info->hook_id = hook_id;
 
 	g_hash_table_replace (priv->registered_folders,
@@ -1502,7 +1529,11 @@ free_worker_thread_info (gpointer data, gpointer user_data)
 	/* Ownership was transfered to us in try_again */
 	free_introduction_info (winfo->intro_info);
 	camel_db_close (winfo->cdb_r);
+#ifdef HAVE_EDS_2_31_2
+	g_object_unref (winfo->store);
+#else
 	camel_object_unref (winfo->store);
+#endif
 	camel_folder_info_free (winfo->iter);
 	g_free (winfo);
 }
@@ -1557,7 +1588,11 @@ on_got_folderinfo_introduce (CamelStore *store,
 
 	/* Ownership of these is transfered in try_again */
 
+#ifdef HAVE_EDS_2_31_2
+	g_object_ref (store);
+#else
 	camel_object_ref (store);
+#endif
 	info->store = store;
 	/* This apparently creates a thread */
 	info->cdb_r = camel_db_clone (store->cdb_r, NULL);
@@ -1600,24 +1635,20 @@ introduce_account_to (TrackerEvolutionPlugin *self,
 {
 	CamelProvider *provider;
 	CamelStore *store;
-	CamelException ex;
 	char *uri, *account_uri, *ptr;
 	IntroductionInfo *intro_info;
 
 	if (!account->enabled || !(uri = account->source->url))
 		return;
 
-	camel_exception_init (&ex);
-	if (!(provider = camel_provider_get(uri, &ex))) {
-		camel_exception_clear (&ex);
+	if (!(provider = camel_provider_get(uri, NULL))) {
 		return;
 	}
 
 	if (!(provider->flags & CAMEL_PROVIDER_IS_STORAGE))
 		return;
 
-	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
-		camel_exception_clear (&ex);
+	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, NULL))) {
 		return;
 	}
 
@@ -1638,7 +1669,11 @@ introduce_account_to (TrackerEvolutionPlugin *self,
 
 	mail_get_folderinfo (store, NULL, on_got_folderinfo_introduce, intro_info);
 
+#ifdef HAVE_EDS_2_31_2
+	g_object_unref (store);
+#else
 	camel_object_unref (store);
+#endif
 
 }
 
@@ -1699,57 +1734,57 @@ register_client_second_half (ClientRegistry *info)
 }
 
 static void
-on_register_client_qry (GPtrArray *results,
-                        GError    *error,
-                        gpointer   user_data)
+on_register_client_qry (GObject *source_object,
+                        GAsyncResult *res,
+                        gpointer user_data)
 {
+	TrackerSparqlCursor *cursor;
 	ClientRegistry *info = user_data;
 	TrackerEvolutionPluginPrivate *priv = TRACKER_EVOLUTION_PLUGIN_GET_PRIVATE (info->self);
-	guint i;
+	GError *error = NULL;
+
+	cursor = tracker_sparql_connection_query_finish (TRACKER_SPARQL_CONNECTION (source_object),
+	                                                 res, &error);
 
 	if (error) {
 		g_warning ("%s\n", error->message);
 		g_error_free (error);
 		g_slice_free (ClientRegistry, info);
+		if (cursor) {
+			g_object_unref (cursor);
+		}
 		return;
 	}
 
-	if (!results) {
+	if (!tracker_sparql_cursor_next (cursor, NULL, NULL)) {
 		if (priv->resuming) {
 			info->last_checkout = priv->last_time;
+		} else if (priv->resuming && priv->last_time != 0) {
+			info->last_checkout = priv->last_time;
 		} else {
 			info->last_checkout = 0;
 		}
 	} else {
-		if (results->len == 0 && priv->resuming && priv->last_time != 0) {
-			info->last_checkout = priv->last_time;
-		} else {
-			if (results->len == 0) {
-				info->last_checkout = 0;
-			} else {
-				for (i = 0; i < results->len; i++) {
-					const gchar **str = g_ptr_array_index (results, i);
-					GError *new_error = NULL;
+		do {
+			const gchar *str = tracker_sparql_cursor_get_string (cursor, 0, NULL);
+			GError *new_error = NULL;
 
-					info->last_checkout = (guint64) tracker_string_to_date (str[0], NULL, &new_error);
+			info->last_checkout = (guint64) tracker_string_to_date (str, NULL, &new_error);
 
-					if (new_error) {
-						g_warning ("%s", new_error->message);
-						g_error_free (error);
-						g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-						g_ptr_array_free (results, TRUE);
-						return;
-					}
-
-					break;
-				}
+			if (new_error) {
+				g_warning ("%s", new_error->message);
+				g_error_free (error);
+				g_object_unref (cursor);
+				return;
 			}
-		}
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
+
+			break;
+		} while (tracker_sparql_cursor_next (cursor, NULL, NULL));
 	}
 
 	register_client_second_half (info);
+
+	g_object_unref (cursor);
 }
 
 static void
@@ -1759,7 +1794,7 @@ register_client (TrackerEvolutionPlugin *self)
 	ClientRegistry *info;
 	const gchar *query;
 
-	if (!priv->client) {
+	if (!priv->connection) {
 		return;
 	}
 
@@ -1772,9 +1807,10 @@ register_client (TrackerEvolutionPlugin *self)
 	query = "SELECT ?c "
 		"WHERE { <" DATASOURCE_URN "> nie:contentLastModified ?c }";
 
-	tracker_resources_sparql_query_async (priv->client, query,
-	                                      on_register_client_qry,
-	                                      info);
+	tracker_sparql_connection_query_async (priv->connection, query,
+	                                       NULL,
+	                                       on_register_client_qry,
+	                                       info);
 }
 
 
@@ -1799,7 +1835,11 @@ on_folder_deleted (CamelStore *store,
 
 static void
 on_folder_renamed (CamelStore *store,
+#ifdef HAVE_EDS_2_31_2
+		   gchar *old_name,
+#else
                    CamelRenameInfo *info,
+#endif
                    StoreRegistry *registry)
 {
 	unregister_account (registry->self, registry->account);
@@ -1817,7 +1857,11 @@ store_registry_new (gpointer co,
 	registry->store = co;
 	registry->account = account; /* weak */
 	registry->self = self; /* weak */
+#ifdef HAVE_EDS_2_31_2
+	g_object_ref (co);
+#else
 	camel_object_ref (co);
+#endif
 
 	return registry;
 }
@@ -1825,8 +1869,16 @@ store_registry_new (gpointer co,
 static void
 store_registry_free (StoreRegistry *registry)
 {
+#ifdef HAVE_EDS_2_31_2
+	g_signal_handler_disconnect (registry->store, registry->hook_id);
+#else
 	camel_object_remove_event (registry->store, registry->hook_id);
+#endif
+#ifdef HAVE_EDS_2_31_2
+	g_object_unref (registry->store);
+#else
 	camel_object_unref (registry->store);
+#endif
 	g_slice_free (StoreRegistry, registry);
 }
 
@@ -1854,27 +1906,45 @@ on_got_folderinfo_register (CamelStore *store,
 
 	/* Hook up catching folder changes in the store */
 	registry = store_registry_new (store, account, self);
+#ifdef HAVE_EDS_2_31_2
+	hook_id = g_signal_connect (store, "folder-created",
+				    G_CALLBACK (on_folder_created),
+				    registry);
+#else
 	hook_id = camel_object_hook_event (store, "folder_created",
 	                                   CAMEL_CALLBACK (on_folder_created),
 	                                   registry);
+#endif
 	registry->hook_id = hook_id;
 	g_hash_table_replace (priv->registered_stores,
 	                      GINT_TO_POINTER (hook_id),
 	                      registry);
 
 	registry = store_registry_new (store, account, self);
+#ifdef HAVE_EDS_2_31_2
+	hook_id = g_signal_connect (store, "folder-renamed",
+				    G_CALLBACK (on_folder_renamed),
+				    registry);
+#else
 	hook_id = camel_object_hook_event (store, "folder_renamed",
 	                                   CAMEL_CALLBACK (on_folder_renamed),
 	                                   registry);
+#endif
 	registry->hook_id = hook_id;
 	g_hash_table_replace (priv->registered_stores,
 	                      GINT_TO_POINTER (hook_id),
 	                      registry);
 
 	registry = store_registry_new (store, account, self);
+#ifdef HAVE_EDS_2_31_2
+	hook_id = g_signal_connect (store, "folder-deleted",
+				    G_CALLBACK (on_folder_deleted),
+				    registry);
+#else
 	hook_id = camel_object_hook_event (store, "folder_deleted",
 	                                   CAMEL_CALLBACK (on_folder_deleted),
 	                                   registry);
+#endif
 	registry->hook_id = hook_id;
 	g_hash_table_replace (priv->registered_stores,
 	                      GINT_TO_POINTER (hook_id),
@@ -1899,24 +1969,20 @@ register_account (TrackerEvolutionPlugin *self,
 {
 	CamelProvider *provider;
 	CamelStore *store;
-	CamelException ex;
 	char *uri;
 	RegisterInfo *reg_info;
 
 	if (!account->enabled || !(uri = account->source->url))
 		return;
 
-	camel_exception_init (&ex);
-	if (!(provider = camel_provider_get(uri, &ex))) {
-		camel_exception_clear (&ex);
+	if (!(provider = camel_provider_get(uri, NULL))) {
 		return;
 	}
 
 	if (!(provider->flags & CAMEL_PROVIDER_IS_STORAGE))
 		return;
 
-	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
-		camel_exception_clear (&ex);
+	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, NULL))) {
 		return;
 	}
 
@@ -1931,7 +1997,11 @@ register_account (TrackerEvolutionPlugin *self,
 	/* Get the account's folder-info and register it asynchronously */
 	mail_get_folderinfo (store, NULL, on_got_folderinfo_register, reg_info);
 
+#ifdef HAVE_EDS_2_31_2
+	g_object_unref (store);
+#else
 	camel_object_unref (store);
+#endif
 }
 
 static gboolean
@@ -1970,22 +2040,18 @@ unregister_account (TrackerEvolutionPlugin *self,
 {
 	CamelProvider *provider;
 	CamelStore *store;
-	CamelException ex;
 	char *uri = account->source->url;
 	RegisterInfo *reg_info;
 
 
-	camel_exception_init (&ex);
-	if (!(provider = camel_provider_get(uri, &ex))) {
-		camel_exception_clear (&ex);
+	if (!(provider = camel_provider_get(uri, NULL))) {
 		return;
 	}
 
 	if (!(provider->flags & CAMEL_PROVIDER_IS_STORAGE))
 		return;
 
-	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
-		camel_exception_clear (&ex);
+	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, NULL))) {
 		return;
 	}
 
@@ -1998,7 +2064,11 @@ unregister_account (TrackerEvolutionPlugin *self,
 	/* Get the account's folder-info and unregister asynchronously */
 	mail_get_folderinfo (store, NULL, on_got_folderinfo_unregister, reg_info);
 
+#ifdef HAVE_EDS_2_31_2
+	g_object_unref (store);
+#else
 	camel_object_unref (store);
+#endif
 }
 
 static void
@@ -2094,10 +2164,10 @@ name_owner_changed_cb (DBusGProxy *proxy,
 
 	if (g_strcmp0 (name, TRACKER_SERVICE) == 0) {
 		 if (tracker_is_empty_string (new_owner) && !tracker_is_empty_string (old_owner)) {
-			if (priv->client) {
-				TrackerClient *client = priv->client;
+			if (priv->connection) {
+				TrackerSparqlConnection *connection = priv->connection;
 
-				priv->client = NULL; 
+				priv->connection = NULL; 
 
 				if (sparql_pool) {
 					ThreadPool *pool = sparql_pool;
@@ -2111,13 +2181,13 @@ name_owner_changed_cb (DBusGProxy *proxy,
 					thread_pool_destroy (pool);
 				}
 
-				g_object_unref (client);
+				g_object_unref (connection);
 			}
 		}
 
 		if (tracker_is_empty_string (old_owner) && !tracker_is_empty_string (new_owner)) {
-			if (!priv->client) {
-				priv->client = tracker_client_new (0, G_MAXINT);
+			if (!priv->connection) {
+				priv->connection = tracker_sparql_connection_get (NULL);
 			}
 			register_client (user_data);
 		}
@@ -2208,10 +2278,10 @@ tracker_evolution_plugin_finalize (GObject *plugin)
 
 	g_object_unref (priv->accounts);
 
-	if (priv->client) {
-		TrackerClient *client = priv->client;
+	if (priv->connection) {
+		TrackerSparqlConnection *connection = priv->connection;
 
-		priv->client = NULL;
+		priv->connection = NULL;
 
 		if (sparql_pool) {
 			ThreadPool *pool = sparql_pool;
@@ -2225,7 +2295,7 @@ tracker_evolution_plugin_finalize (GObject *plugin)
 			thread_pool_destroy (pool);
 		}
 
-		g_object_unref (client);
+		g_object_unref (connection);
 	}
 
 	if (priv->dbus_proxy) {
@@ -2233,7 +2303,7 @@ tracker_evolution_plugin_finalize (GObject *plugin)
 	}
 
 	if (priv->connection) {
-		dbus_g_connection_unref (priv->connection);
+		dbus_g_connection_unref (priv->dbusconnection);
 	}
 
 	G_OBJECT_CLASS (tracker_evolution_plugin_parent_class)->finalize (plugin);
@@ -2263,7 +2333,7 @@ tracker_evolution_plugin_init (TrackerEvolutionPlugin *plugin)
 	EIterator *it;
 	GError *error = NULL;
 
-	priv->client = NULL;
+	priv->connection = NULL;
 	priv->last_time = 0;
 	priv->resuming = FALSE;
 	priv->paused = FALSE;
@@ -2273,13 +2343,13 @@ tracker_evolution_plugin_init (TrackerEvolutionPlugin *plugin)
 	priv->registered_stores = NULL;
 	priv->registered_clients = NULL;
 
-	priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+	priv->dbusconnection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 
 	if (error) {
 		goto error_handler;
 	}
 
-	priv->dbus_proxy = dbus_g_proxy_new_for_name (priv->connection,
+	priv->dbus_proxy = dbus_g_proxy_new_for_name (priv->dbusconnection,
 	                                              DBUS_SERVICE_DBUS,
 	                                              DBUS_PATH_DBUS,
 	                                              DBUS_INTERFACE_DBUS);
@@ -2333,8 +2403,8 @@ miner_started (TrackerMiner *miner)
 {
 	TrackerEvolutionPluginPrivate *priv = TRACKER_EVOLUTION_PLUGIN_GET_PRIVATE (miner);
 
-	if (!priv->client) {
-		priv->client = tracker_client_new (0, G_MAXINT);
+	if (!priv->connection) {
+		priv->connection = tracker_sparql_connection_get (NULL);
 	}
 
 	dbus_g_proxy_begin_call (priv->dbus_proxy, "ListNames",
@@ -2344,7 +2414,7 @@ miner_started (TrackerMiner *miner)
 	                         G_TYPE_INVALID,
 	                         G_TYPE_INVALID);
 
-	g_object_set (miner,  "progress", 0.0,  "status", _("Initializing"), NULL);
+	g_object_set (miner,  "progress", 0.0,  "status", "Initializing", NULL);
 }
 
 static void
@@ -2369,10 +2439,10 @@ miner_paused (TrackerMiner *miner)
 	priv->paused = TRUE;
 	priv->last_time = 0;
 
-	if (priv->client) {
-		TrackerClient *client = priv->client;
+	if (priv->connection) {
+		TrackerSparqlConnection *connection = priv->connection;
 
-		priv->client = NULL;
+		priv->connection = NULL;
 
 		if (sparql_pool) {
 			ThreadPool *pool = sparql_pool;
@@ -2386,7 +2456,7 @@ miner_paused (TrackerMiner *miner)
 			thread_pool_destroy (pool);
 		}
 
-		g_object_unref (client);
+		g_object_unref (connection);
 
 		/* By setting this to NULL, events will still be catched by our
 		 * handlers, but the send_sparql_* calls will just ignore it.
@@ -2432,11 +2502,11 @@ miner_resumed (TrackerMiner *miner)
 	priv->total_popped = 0;
 	priv->of_total = 0;
 
-	if (!priv->client) {
-		priv->client = tracker_client_new (0, G_MAXINT);
+	if (!priv->connection) {
+		priv->connection = tracker_sparql_connection_get (NULL);
 	}
 
-	g_object_set (miner,  "progress", 0.0,  "status", _("Resuming"), NULL);
+	g_object_set (miner,  "progress", 0.0,  "status", _("Processing…"), NULL);
 
 	dbus_g_proxy_begin_call (priv->dbus_proxy, "ListNames",
 	                         list_names_reply_cb,
